@@ -30,11 +30,12 @@ struct _GdbInstance {
     gint display_array_size;
     gint num_displays;
 
-    gboolean file_loaded;
-    gboolean is_running;    /* Loaded and ready to execute */
-    gboolean is_executing;  /* Currently executing */
-    gboolean at_prompt;
-    gboolean at_query;
+    gboolean file_loaded : 1;
+    gboolean is_running : 1;    /* Loaded and ready to execute */
+    gboolean is_executing : 1;  /* Currently executing */
+    gboolean is_attached : 1;   /* Attached to an external process */
+    gboolean at_prompt : 1;
+    gboolean at_query : 1;
 
     gint listener_id;
     double gdb_ver;
