@@ -22,18 +22,21 @@
 
 #include "libdryad.h"
 #include <string.h>
+#include <ctype.h>
+
 #include "gdf-libdryad-debugger.h"
 
+static void read_type (GdbInstance *inst, GDF_Symbol *sym);
 static void read_value (GdbInstance *inst, GDF_Symbol *sym);
-static void check_type (GDF_Symbol *sym);
 static void expand_pointer (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
 			    GDF_Symbol *sym);
 static void expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 			      GDF_Symbol *sym);
-
+static void expand_array (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
+			  GDF_Symbol *sym);
+static void destroy_children (GDF_Symbol *sym);
 static GDF_SymbolSequence *init_seq (void);
 static void grow_seq (GDF_SymbolSequence *seq, int amount);
-
 void
 gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
 {
@@ -64,7 +67,7 @@ gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
 		int levels = 1;
 		while (levels > 0) {
 		    line = gdb_read_line (inst);
-		    if (!strncmp (line, "{", 1))
+		    if (!strncmp (line, "{", 1) || !strncmp (line, ", {", 3))
 			levels++;
 		    if (!strncmp (line, "}", 1))
 			levels--;
@@ -74,7 +77,6 @@ gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
 	line = gdb_read_line (inst);
     }
 
-
     /* Add all the locals to the symbol set */
     names = g_list_reverse (names);
 
@@ -91,16 +93,11 @@ gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
 void
 gdb_get_expression (GdbInstance *inst, GDF_Symbol *sym, const char *expr)
 {
-    char *cmd_str;
-    
-    cmd_str = g_strdup_printf ("output %s", expr);
-    gdb_execute_command (inst, cmd_str);
-    g_free (cmd_str);
-    
     sym->name = CORBA_string_dup (expr);
     sym->expression = CORBA_string_dup (expr);
+
+    read_type (inst, sym);
     read_value (inst, sym);
-    check_type (sym);
 }
 
 void
@@ -117,6 +114,8 @@ gdb_expand_symbol (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
 	case GDF_TYPE_STRUCT :
 	    expand_structure (inst, ss, sym);
 	    break;
+	case GDF_TYPE_ARRAY :
+	    expand_array (inst, ss, sym);
 	default:
 	    return;
 	}
@@ -129,8 +128,8 @@ static void
 update_seq (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 	    GDF_SymbolSequence *seq) 
 {
-    char *cmd_str;
     char *old_value;
+    char *old_typename;
     GDF_Symbol *sym;
     CORBA_boolean was_expandable;
 
@@ -138,17 +137,29 @@ update_seq (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
 
     for (i = 0; i < seq->_length; i++) {
 	sym = &seq->_buffer[i];
-	
-	cmd_str = g_strdup_printf ("output %s", sym->expression);
-	gdb_execute_command (inst, cmd_str);
-	g_free (cmd_str);
-	
+
 	old_value = sym->value;
-	
+	sym->value = NULL;
+	old_typename = sym->typename;
+	sym->typename = NULL;
 	was_expandable = sym->expandable;
 
+	read_type (inst, sym);
 	read_value (inst, sym);
-	check_type (sym);
+	
+	if (strcmp (old_typename, sym->typename)) {
+	    CORBA_any *event_any;
+	    
+	    destroy_children (sym);
+
+	    event_any = gdf_marshal_event_long ("symbol_type_changed", 
+						sym->handle);
+	    
+	    gdf_event_channel_client_push (ss->event_channel,
+					   event_any);
+	    CORBA_free (event_any);
+	}
+	
 	if (strcmp (old_value, sym->value) 
 	    || was_expandable != sym->expandable) {
 	    CORBA_any *event_any;
@@ -162,6 +173,7 @@ update_seq (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
 	}
 	
 	CORBA_free (old_value);
+	CORBA_free (old_typename);
 	
 	update_seq (inst, ss, 
 		    (GDF_SymbolSequence*)sym->children._value);
@@ -204,11 +216,97 @@ gdb_add_expression (GdbInstance *inst,
 }
 
 void
-read_value (GdbInstance *inst, GDF_Symbol *sym) 
+read_type (GdbInstance *inst, GDF_Symbol *sym)
 {
     char *line;
+    char *cmd_str;
 
+    g_assert (sym->expression);
+    
+    cmd_str = g_strdup_printf ("ptype %s", sym->expression);
+    gdb_execute_command (inst, cmd_str);
+    g_free (cmd_str);
+
+    if (sym->typename) 
+	CORBA_free (sym->typename);
+    sym->typename = NULL;
+
+    line = gdb_read_line (inst);
+    while (strcmp (line, "\32\32pre-prompt")) {
+	if (line[0] != '\0' && strcmp (line, "\32\32post-prompt")) {
+	    char *type;
+	    if (!strcmp (line, "\32\32error-begin"))
+		line = gdb_read_line (inst);
+	    if (!strncmp (line, "type = ", strlen ("type = ")))
+		line += 7;
+
+	    type = g_strdup (line);
+
+	    if (line[strlen (line) - 1] == '{') {
+		int depth = 1;
+		char *newtype;
+		while (depth) {
+		    line = gdb_read_line (inst);
+		    while (*line) {
+			if (*line == '{')
+			    depth++;
+			if (*line++ == '}')
+			    depth--;
+		    }
+		}
+		
+		newtype = g_strdup_printf ("%s%s", type, line);
+		g_free (type);
+		type = newtype;
+	    }
+
+	    sym->typename = CORBA_string_dup (type);
+	    g_free (type);
+	    
+	    break;
+	}
+	line = gdb_read_line (inst);
+    }
+
+    g_assert (sym->typename);
+    
+    sym->expandable = FALSE;
+    sym->type = GDF_TYPE_NORMAL;
+    if (!strncmp (sym->typename, "struct", strlen ("struct"))) {
+	sym->expandable = TRUE;
+	sym->type = GDF_TYPE_STRUCT;
+    }
+    if (sym->typename[strlen (sym->typename) - 1] == ']') {
+	sym->expandable = TRUE;
+	sym->type = GDF_TYPE_ARRAY;
+    }
+    if (sym->typename[strlen (sym->typename) - 1] == '*') {
+	sym->expandable = TRUE;
+	sym->type = GDF_TYPE_PTR;
+    }
+}
+
+void
+read_value (GdbInstance *inst, GDF_Symbol *sym) 
+{
+    char *cmd_str;
+    char *line;
+
+    g_assert (!sym->value);
     sym->value = NULL;
+
+    if (sym->type == GDF_TYPE_STRUCT) {
+	sym->value = CORBA_string_dup ("{...}");
+	return;
+    } else if (sym->type == GDF_TYPE_ARRAY) {
+	sym->value = CORBA_string_dup ("[...]");
+	return;
+    }
+
+    cmd_str = g_strdup_printf ("output %s", sym->expression);
+    gdb_execute_command (inst, cmd_str);
+    g_free (cmd_str);
+
     line = gdb_read_line (inst);
     while (strcmp (line, "\32\32pre-prompt")) {
 	if (!strncmp(line, "\32\32value-begin", strlen("\32\32value-begin"))) {
@@ -221,9 +319,9 @@ read_value (GdbInstance *inst, GDF_Symbol *sym)
 	    if (strcmp (line, "\32\32error-begin")) {
 		sym->value = CORBA_string_dup (line);
 	    } else {
+		line = gdb_read_line (inst);
 		while (strcmp (line, "\32\32error")) {
 		    char *new;
-		    line = gdb_read_line (inst);
 		    new = g_strdup_printf ("%s%s",
 					   sym->value ? sym->value : "", 
 					   line);
@@ -232,34 +330,25 @@ read_value (GdbInstance *inst, GDF_Symbol *sym)
 		    }
 		    sym->value = CORBA_string_dup (new);
 		    g_free (new);
+		    line = gdb_read_line (inst);
 		}
 		break;
 	    }
-
-	    if (!strncmp (line, "{", 1)) {
-		int levels = 1;
-		while (levels > 0) {
-		    line = gdb_read_line (inst);
-		    if (!strncmp (line, "{", 1))
-			levels++;
-		    if (!strncmp (line, "}", 1))
-			levels--;
-		}
-	    }
 	    break;
 	}
 	if (!strcmp (line, "\32\32error-begin")) {
+	    line = gdb_read_line (inst);
 	    while (strcmp (line, "\32\32error")) {
 		char *new;
-		line = gdb_read_line (inst);
 		new = g_strdup_printf ("%s%s",
 				       sym->value ? sym->value : "", 
-					   line);
+				       line);
 		if (sym->value) {
 		    CORBA_free (sym->value);
 		}
 		sym->value = CORBA_string_dup (new);
-		g_free (new);
+		g_free (new);	
+		line = gdb_read_line (inst);
 	    }
 	}
 	line = gdb_read_line (inst);
@@ -267,36 +356,6 @@ read_value (GdbInstance *inst, GDF_Symbol *sym)
     g_assert (sym->value != NULL);
 }
 
-void
-check_type (GDF_Symbol *sym)
-{
-    /* Check for pointer types */
-    char *val = sym->value;
-    
-    sym->expandable = FALSE;
-    sym->type = GDF_TYPE_NORMAL;
-
-    if (val[0] == '(') {
-	while (*val != ')') {
-            if (*val++ == '*') {  /* FIXME: This is C/C++ specific */
-		sym->expandable = TRUE;
-		sym->type = GDF_TYPE_PTR;
-		return;
-	    }
-	}
-    }
-
-    val = sym->value;
-    if (val[0] == '{') {
-	sym->expandable = TRUE;
-	sym->type = GDF_TYPE_STRUCT;
-	return;
-    }
-    /* FIXME: Check for array types */
-
-    sym->type = GDF_TYPE_NORMAL;
-}
-
 static void
 add_child_expression (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 		      GDF_Symbol *sym, const char *name, const char *expr)
@@ -324,41 +383,71 @@ expand_pointer (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
     g_free (new_name);
 }
 
+static char *
+read_child_from_ptype (GdbInstance *inst)
+{
+    char *name;
+    char *work;
+    char *line;
+
+    line = gdb_read_line (inst);
+
+    if (line[0] == '}') 
+	return NULL;
+
+    /* Clean out the trailing ;, and any array indices if necessary */
+    work = line + (strlen (line) - 1);
+    if (*work == '{') {
+	do {
+	    line = gdb_read_line (inst);
+	    while (isspace (*line++))
+		;    
+	    line--;
+	} while (*line != '}');
+    } 
+
+    work = strrchr (line, ';');
+    if (work) {
+	*work-- = '\0';
+    } else {
+	work = line + (strlen(line) - 1);
+    }    
+
+    while (*work == ']') {
+	while (*work-- != '[') 
+	    ;
+	*(work + 1) = '\0';
+    }
+
+    while (*work != '*' && *work-- != ' ')
+	;
+    name = work + 1;
+    return g_strdup (name);
+}
+
 void
 expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 		  GDF_Symbol *sym)
 {
-    GDF_SymbolSequence *children;
     char *expr;
     char *cmd_str;
     char *line;
+    char *name;
     GList *names = NULL;
     GList *i;
 
-    cmd_str = g_strdup_printf ("output %s", sym->expression);
+    cmd_str = g_strdup_printf ("ptype %s", sym->expression);
     gdb_execute_command (inst, cmd_str);
     g_free (cmd_str);
-    
-    children = sym->children._value;
-    line = gdb_read_line (inst);
-    while (strcmp (line, "\32\32pre-prompt")) {
-	if (!strncmp (line, "\32\32field-begin", 
-		      strlen ("\32\32field-begin"))) {
-	    line = gdb_read_line (inst);
-	    names = g_list_prepend (names, g_strdup (line));
 
-	    if (!strncmp (line, "{", 1)) {
-		int levels = 1;
-		while (levels > 0) {
-		    line = gdb_read_line (inst);
-		    if (!strncmp (line, "{", 1))
-			levels++;
-		    if (!strncmp (line, "}", 1))
-			levels--;
-		}
-	    }
-	}
+    do {
 	line = gdb_read_line (inst);
+    } while (strncmp (line, "type = ", strlen ("type = ")));
+
+    name = read_child_from_ptype (inst);
+    while (name) {
+	names = g_list_prepend (names, name);
+	name = read_child_from_ptype (inst);
     }
 
     names = g_list_reverse (names);
@@ -373,6 +462,48 @@ expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
     g_list_free (names); 
 }
 
+void
+expand_array (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
+	      GDF_Symbol *sym)
+{
+    char *typecpy;
+    char *p;
+    int size;
+    int i;
+    
+    typecpy = g_strdup (sym->typename);
+
+    /* isolate the first index size */
+    p = strchr (typecpy, ']');
+    g_assert (p);
+    *p = '\0';
+    
+    p = strchr (typecpy, '[');
+    g_assert (p);
+
+    size = atoi (p + 1);
+    
+    g_free (typecpy);
+
+    for (i = 0; i < size; i++) {
+	char *expr;
+	char *name;
+ 	expr = g_strdup_printf ("(%s)[%d]", sym->expression, i);
+	name = g_strdup_printf ("[%d]", i);
+	add_child_expression (inst, ss, sym, name, expr);
+	g_free (expr);
+	g_free (name);
+    }
+}
+
+static void
+destroy_children (GDF_Symbol *sym)
+{
+    CORBA_free (sym->children._value);
+    sym->children._value = init_seq ();
+    sym->expanded = CORBA_FALSE;
+}
+
 static GDF_SymbolSequence *
 init_seq (void)
 {
@@ -410,6 +541,7 @@ grow_seq (GDF_SymbolSequence *seq, int amount)
 	    sym->expression = old_sym->expression;
 	    sym->type = old_sym->type;
 	    sym->value = old_sym->value;
+	    sym->typename = old_sym->typename;
 	    sym->expandable = old_sym->expandable;
 	    sym->expanded = old_sym->expanded;
 	    sym->children = old_sym->children;
