@@ -43,6 +43,7 @@ static void gdf_libdryad_debugger_class_init (GdfLibdryadDebuggerClass *class);
 static void gdf_libdryad_debugger_init (BonoboObject *object);
 static void connect_signals (GdfLibdryadDebugger *dbg);
 static void process_frame_change (GdfLibdryadDebugger *dbg);
+static void update_symsets (GdfLibdryadDebugger *dbg, gboolean include_locals);static void new_locals (GdfLibdryadDebugger *dbg);
 
 static BonoboObjectClass *parent_class;
 static POA_GDF_Debugger__epv libdryad_debugger_epv;
@@ -77,7 +78,7 @@ gdf_libdryad_debugger_new (void)
 			     objref);
     
     debugger->gdb_inst = NULL;
-    debugger->state = GDF_Debugger_NONE;
+    debugger->state = 0;
     debugger->event_channel = gdf_event_channel_client_new ();
 
     debugger->pending_exit = -1;
@@ -141,7 +142,7 @@ static void
 gdf_libdryad_debugger_destroy (GtkObject *object) 
 {
     GdfLibdryadDebugger *dbg = GDF_LIBDRYAD_DEBUGGER (object);
-    
+
     if (dbg->gdb_inst)
 	gdb_destroy_instance (dbg->gdb_inst);
 
@@ -151,7 +152,7 @@ gdf_libdryad_debugger_destroy (GtkObject *object)
     }
 }
 
-static GDF_Debugger_State
+static GDF_Debugger_StateFlag
 impl_get_state (PortableServer_Servant servant, CORBA_Environment *ev)
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
@@ -171,7 +172,7 @@ impl_load_binary (PortableServer_Servant servant,
 	if (dbg->gdb_inst->file_loaded) 
 	    gdb_unload_file (dbg->gdb_inst);
 	
-	dbg->state = GDF_Debugger_NONE;
+	dbg->state |= GDF_Debugger_BINARY_LOADED;
 	
 	event_any = gdf_marshal_event_none ("program_unloaded");
 	gdf_event_channel_client_push (dbg->event_channel, event_any);
@@ -209,7 +210,8 @@ impl_unload_binary (PortableServer_Servant servant, CORBA_Environment *ev)
     if (dbg->gdb_inst) {
 	if (dbg->gdb_inst->file_loaded) 
 	    gdb_unload_file (dbg->gdb_inst);
-	dbg->state = GDF_Debugger_NONE;
+
+	dbg->state &= ~(GDF_Debugger_BINARY_LOADED);
 	
 	event_any = gdf_marshal_event_none ("program_unloaded");
 	gdf_event_channel_client_push (dbg->event_channel, event_any);
@@ -219,25 +221,68 @@ impl_unload_binary (PortableServer_Servant servant, CORBA_Environment *ev)
     dbg->gdb_inst = NULL;
 }
 
-/* "Who's binary?" - Anne Wooten */
-static CORBA_boolean
-impl__get_binary_loaded (PortableServer_Servant servant, 
-			 CORBA_Environment *ev)
-{
-    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-
-    if (dbg->gdb_inst && dbg->gdb_inst->file_loaded)
-	return CORBA_TRUE;
-    else
-	return CORBA_FALSE;
-}
-
 static void
 impl_attach (PortableServer_Servant servant, 
 	     CORBA_short pid,
 	     CORBA_Environment *ev)
 {
-    /* not implemented */
+    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    GdbAttachResult res;
+    CORBA_any *event_any;
+
+    res = gdb_attach (dbg->gdb_inst, (int)pid);
+
+    switch (res) {
+    case GDB_ATTACH_OK :
+	dbg->state |= GDF_Debugger_ATTACHED;
+	dbg->state |= GDF_Debugger_TARGET_EXISTS;
+	event_any = gdf_marshal_event_none ("attached");
+	gdf_event_channel_client_push (dbg->event_channel, event_any);
+	CORBA_free (event_any);
+	break;
+    case GDB_ATTACH_ALREADY_RUNNING :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	break;
+    case GDB_ATTACH_DOESNT_EXIST :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_DoesntExist, NULL);
+	break;
+    case GDB_ATTACH_ACCESS_DENIED :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_SecurityException, NULL);
+	break;
+    case GDB_ATTACH_MISC :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_UnknownError, NULL);
+	break;
+    }
+}
+
+static void
+impl_detach (PortableServer_Servant servant,
+	     CORBA_Environment *ev)
+{
+    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    CORBA_any *event_any;
+
+    if ((dbg->state & GDF_Debugger_ATTACHED) != GDF_Debugger_ATTACHED) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+    dbg->state &= ~GDF_Debugger_ATTACHED;
+    dbg->state &= ~GDF_Debugger_TARGET_EXISTS;
+    
+    gdb_detach (dbg->gdb_inst);
+    new_locals (dbg);
+    update_symsets (dbg, TRUE);
+    event_any = gdf_marshal_event_none ("detached");
+    gdf_event_channel_client_push (dbg->event_channel, event_any);
+    CORBA_free (event_any);
+    event_any = gdf_marshal_event_long ("exited", -1);
+    gdf_event_channel_client_push (dbg->event_channel, event_any);
+    CORBA_free (event_any);
 }
 
 static void
@@ -247,7 +292,7 @@ impl_execute (PortableServer_Servant servant,
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
     g_return_if_fail (dbg->gdb_inst != NULL);
-    
+
     gdb_run (dbg->gdb_inst);
 }
 
@@ -526,7 +571,7 @@ impl_get_frame (PortableServer_Servant servant,
 
     gdf_frame = GDF_StackFrame__alloc ();
 
-    if (dbg->state != GDF_Debugger_NONE) {
+    if ((dbg->state & GDF_Debugger_TARGET_EXISTS) == GDF_Debugger_TARGET_EXISTS) {
 	frame = gdb_get_frame (dbg->gdb_inst, id);
 	
 	gdf_frame->id = frame->id;
@@ -568,7 +613,7 @@ impl_get_backtrace (PortableServer_Servant servant, CORBA_Environment *ev)
     stack = GDF_Stack__alloc ();
     CORBA_sequence_set_release (stack, CORBA_TRUE);
 
-    if (dbg->state == GDF_Debugger_NONE) {
+    if ((dbg->state & GDF_Debugger_TARGET_EXISTS) != GDF_Debugger_TARGET_EXISTS) {
 	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 			     ex_GDF_Debugger_NotRunning, NULL);
 	return stack;
@@ -656,7 +701,7 @@ impl_step_out (PortableServer_Servant servant, CORBA_Environment *ev)
 
 /* Symbol watching */
 
-static void
+void
 update_symsets (GdfLibdryadDebugger *dbg, gboolean include_locals)
 {
     GList *i;
@@ -670,7 +715,7 @@ update_symsets (GdfLibdryadDebugger *dbg, gboolean include_locals)
     }
 }
 
-static void
+void
 new_locals (GdfLibdryadDebugger *dbg)
 {
     gdb_get_locals (dbg->gdb_inst, dbg->locals);
@@ -703,7 +748,6 @@ impl_get_event_channel (PortableServer_Servant servant,
 {
     CORBA_Object ret;
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-    GDF_TRACE ();
     ret = bonobo_object_corba_objref (BONOBO_OBJECT (dbg->event_channel));
     return CORBA_Object_duplicate (ret, ev);
 }
@@ -731,8 +775,8 @@ init_libdryad_debugger_corba_class (void)
     libdryad_debugger_epv.get_state = impl_get_state;
     libdryad_debugger_epv.load_binary = impl_load_binary;
     libdryad_debugger_epv.unload_binary = impl_unload_binary;
-    libdryad_debugger_epv._get_binary_loaded = impl__get_binary_loaded;
     libdryad_debugger_epv.attach = impl_attach;
+    libdryad_debugger_epv.detach = impl_detach;
     libdryad_debugger_epv.execute = impl_execute;
     libdryad_debugger_epv.load_corefile = impl_load_corefile;
     libdryad_debugger_epv.get_sources = impl_get_sources;
@@ -794,6 +838,7 @@ static void
 execution_starting_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
+    dbg->state |= GDF_Debugger_TARGET_EXISTS;
 
     event_any = gdf_marshal_event_none ("started");
     gdf_event_channel_client_push (dbg->event_channel, event_any);
@@ -804,7 +849,7 @@ static void
 execution_running_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
-    dbg->state = GDF_Debugger_RUNNING;
+    dbg->state |= GDF_Debugger_TARGET_EXECUTING;
 
     event_any = gdf_marshal_event_none ("running");
     gdf_event_channel_client_push (dbg->event_channel, event_any);
@@ -851,13 +896,15 @@ static void
 execution_stopped_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
-    dbg->state = GDF_Debugger_STOPPED;
+    dbg->state &= ~GDF_Debugger_TARGET_EXECUTING;
 
     event_any = gdf_marshal_event_none ("stopped");
     dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
 
     if (dbg->pending_exit != -1) {
-	dbg->state = GDF_Debugger_NONE;
+	dbg->state &= ~GDF_Debugger_TARGET_EXISTS;
+	new_locals (dbg);
+	update_symsets (dbg, TRUE);
 	event_any = gdf_marshal_event_long ("exited", 
 					    (glong)dbg->pending_exit);
 	dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
@@ -883,7 +930,7 @@ static void
 execution_killed_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
-    dbg->state = GDF_Debugger_STOPPED;
+    dbg->state &= ~GDF_Debugger_TARGET_EXECUTING;
 
     g_warning ("libdryad debugger does not send the correct kill signal");
 
