@@ -23,7 +23,7 @@ struct _GdfLibdryadDebugger {
 
     GdbInstance *gdb_inst;
     GdfEventChannelClient *event_channel;
-    GDF_Debugger_State state;
+    GDF_Debugger_StateFlag state;
 
     gchar *tty_name;
     gint pending_exit;
