@@ -102,7 +102,7 @@ static GtkFrameClass *parent_class;
  */
 
 GtkType 
-gdf_source_viewer_get_type ()
+gdf_source_viewer_get_type (void)
 {
     static GtkType type = 0;
 	
@@ -484,7 +484,7 @@ scroll_to_line (GdfSourceViewer *sv, gint line_num)
     GtkAdjustment *adjustment;
     gint pixel;
 	
-    gtk_clist_select_row (sv->priv->clist, line_num - 1, 0);
+    gtk_clist_select_row (GTK_CLIST (sv->priv->clist), line_num - 1, 0);
 
     if (line_is_visible (sv, line_num))
         return;
@@ -511,7 +511,7 @@ set_breakpoint_selected_cb (GtkWidget *widget, GdfSourceViewer *sv)
                                                      "menu_line"));
 	
     gdf_debugger_client_set_breakpoint (sv->priv->dbg,
-                                         sv->source_path, line_num, NULL);
+                                        sv->source_path, line_num, NULL, NULL);
 }
 
 static void
