@@ -35,6 +35,7 @@ struct _GdfVariableViewerPriv {
 
 	/* signal identifiers */
     guint symbol_changed_sig;
+    guint symbol_type_changed_sig;
     guint symbol_set_changed_sig;
 	guint program_unloaded_sig;
 
@@ -87,7 +88,7 @@ static GtkFrameClass *parent_class;
  */
 
 GtkType
-gdf_variable_viewer_get_type ()
+gdf_variable_viewer_get_type (void)
 {
 	static GtkType variable_viewer_type = 0;
 	
@@ -111,7 +112,7 @@ gdf_variable_viewer_get_type ()
 }
 
 GdfVariableViewer *
-gdf_variable_viewer_new ()
+gdf_variable_viewer_new (void)
 {
 	GdfVariableViewer *vv;
 	
@@ -131,8 +132,8 @@ gdf_variable_viewer_view_symbol_set (GdfVariableViewer *vv,
 
     if (vv->priv->dbg) {
         if (!ss) {
-            vv->priv->ss =
-                gdf_debugger_client_allocate_symbol_set (vv->priv->dbg);
+            gdf_debugger_client_allocate_symbol_set (vv->priv->dbg, 
+                                                     &vv->priv->ss);
         } else {
             vv->priv->ss = ss;
         }
@@ -150,8 +151,7 @@ gdf_variable_viewer_view_locals (GdfVariableViewer *vv)
     vv->priv->view_locals = TRUE;
 
     if (vv->priv->dbg) {
-        vv->priv->ss =
-            gdf_debugger_client_get_locals (vv->priv->dbg);
+        gdf_debugger_client_get_locals (vv->priv->dbg, &vv->priv->ss);
     }
     
     /* FIXME: Get the current symbol set */
@@ -204,10 +204,11 @@ gdf_variable_viewer_set_debugger (GdfVariableViewer *vv,
         if (GTK_OBJECT_FLOATING (GTK_OBJECT (dbg))) 
             gtk_object_sink (GTK_OBJECT (dbg));
         if (vv->priv->view_locals) {
-            vv->priv->ss = gdf_debugger_client_get_locals (vv->priv->dbg);
+            gdf_debugger_client_get_locals (vv->priv->dbg, 
+                                            &vv->priv->ss);
         } else {
-            vv->priv->ss = 
-                gdf_debugger_client_allocate_symbol_set (vv->priv->dbg);
+            gdf_debugger_client_allocate_symbol_set (vv->priv->dbg,
+                                                     &vv->priv->ss);
         }
 
         connect_debugger_signals (vv);
@@ -322,7 +323,8 @@ set_arg (GtkObject *object,
     
     switch (arg_id) {
     case ARG_DEBUGGER :
-        gdf_variable_viewer_set_debugger (vv, GTK_VALUE_OBJECT (*arg));
+        gdf_variable_viewer_set_debugger (vv, 
+                                          GTK_VALUE_OBJECT (*arg) ? GDF_DEBUGGER_CLIENT (GTK_VALUE_OBJECT (*arg)) : NULL);
         break;
     case ARG_SHOW_ENTRY :
         if (GTK_VALUE_BOOL (*arg)) {
@@ -477,6 +479,35 @@ symbol_changed_cb (GdfSymbolSetClient *ss, int handle, GdfVariableViewer *vv)
     CORBA_free (sym);
 }
 
+static void
+symbol_type_changed_cb (GdfSymbolSetClient *ss, int handle,
+                        GdfVariableViewer *vv)
+{
+    GDF_Symbol *sym;
+    GtkCTreeNode *node;
+    GtkCTreeNode *child;
+    GtkCTreeRow *row;
+
+    sym = gdf_symbol_set_client_get_symbol (ss, handle);
+    
+    node = g_tree_lookup (vv->priv->nodes, GINT_TO_POINTER (handle));
+    row = GTK_CTREE_ROW (node);
+    
+    /* Remove the old/dummy children */
+    child = GTK_CTREE_ROW (node)->children;
+    while (child) {
+        GtkCTreeNode *next;
+        next = GTK_CTREE_ROW (child)->sibling;
+        gtk_ctree_remove_node (GTK_CTREE (vv->priv->ctree), child);
+        child = next;
+    }
+
+    if (sym->expandable) {
+        add_dummy_child (vv, node);
+    }
+}
+
+
 void
 connect_debugger_signals (GdfVariableViewer *vv)
 {
@@ -503,6 +534,11 @@ connect_symbol_set_signals (GdfVariableViewer *vv)
                             "symbol_changed",
                             GTK_SIGNAL_FUNC (symbol_changed_cb),
                             (gpointer)vv);
+    vv->priv->symbol_type_changed_sig = 
+        gtk_signal_connect (GTK_OBJECT (vv->priv->ss),
+                            "symbol_type_changed",
+                            GTK_SIGNAL_FUNC (symbol_type_changed_cb),
+                            (gpointer)vv);
     vv->priv->symbol_set_changed_sig =
         gtk_signal_connect (GTK_OBJECT (vv->priv->ss),
                             "symbol_set_changed",
@@ -515,6 +551,8 @@ disconnect_symbol_set_signals (GdfVariableViewer *vv)
 {
 	gtk_signal_disconnect (GTK_OBJECT (vv->priv->ss), 
 						   vv->priv->symbol_changed_sig);
+	gtk_signal_disconnect (GTK_OBJECT (vv->priv->ss), 
+						   vv->priv->symbol_type_changed_sig);
     gtk_signal_disconnect (GTK_OBJECT (vv->priv->ss),
                            vv->priv->symbol_set_changed_sig);
 }
@@ -580,10 +618,14 @@ tree_expand_cb (GtkWidget *ctree, GtkCTreeNode *node, gpointer data)
             child = next;
         }
 
+        gtk_clist_freeze (GTK_CLIST (vv->priv->ctree));
+
         for (i = 0; i < seq->_length; i++) {
             add_node (vv, node, &(seq->_buffer[i]));
         }
 
+        gtk_clist_thaw (GTK_CLIST (vv->priv->ctree));
+
         CORBA_free (seq);
     }
 }
