@@ -34,9 +34,7 @@ static void debugger_client_destroy (GdfDebuggerClient *prog);
 static void event_pushed_cb (GdfEventChannelClient *event_channel,
                              CORBA_any *val,
                              GdfDebuggerClient *client);
-static void program_loaded_impl (GdfDebuggerClient *dbg);
-static void program_unloaded_impl (GdfDebuggerClient *dbg);
-
+static GdfDebuggerClientResult exception_to_result (CORBA_Environment *ev);
 
 static GtkObjectClass *parent_class;
 
@@ -54,6 +52,8 @@ enum {
     BREAKPOINT_ENABLED,
     BREAKPOINT_DISABLED,
     BREAKPOINT_DELETED,
+    ATTACHED,
+    DETACHED,
     EXECUTION_STARTED,
     EXECUTION_RUNNING,
     EXECUTION_STOPPED,
@@ -75,7 +75,6 @@ gdf_debugger_client_construct (GdfDebuggerClient *dbg,
     CORBA_Object channel;
     CORBA_Environment ev;
 
-    dbg->program_loaded = FALSE;
     dbg->objref = corba_object;
     dbg->priv = g_new0 (GdfDebuggerClientPrivate, 1);
     dbg->priv->cur_frame = -1;
@@ -125,7 +124,8 @@ gdf_debugger_client_new (const char *iid)
 #endif
     CORBA_exception_init (&ev);
 
-    corba_object = oaf_activate_from_id (iid, OAF_FLAG_IGNORE_EXISTING, 
+    corba_object = oaf_activate_from_id ((const OAF_ActivationID)iid, 
+                                         OAF_FLAG_IGNORE_EXISTING, 
                                          NULL, &ev);    
     
     if (CORBA_Object_is_nil (corba_object, &ev)) {
@@ -166,8 +166,6 @@ gdf_debugger_client_new_from_corba (GDF_Debugger debugger)
 
     Bonobo_Unknown_ref (debugger, &ev);
 
-    ret->program_loaded = GDF_Debugger__get_binary_loaded (debugger, &ev);
-    
     /* FIXME: Check env */
     
     CORBA_exception_free (&ev);
@@ -198,15 +196,14 @@ gdf_debugger_client_get_type (void)
     return type;
 }
 
-GDF_Debugger_State
+GDF_Debugger_StateFlag
 gdf_debugger_client_get_state (GdfDebuggerClient *dbg)
 {
-    GDF_Debugger_State ret;
+    GDF_Debugger_StateFlag ret;
     CORBA_Environment ev;
 
-    g_return_val_if_fail (dbg != NULL, GDF_Debugger_UNKNOWN);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
-                          GDF_Debugger_UNKNOWN);
+    g_return_val_if_fail (dbg != NULL, 0);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 0);
 
     CORBA_exception_init (&ev);
     
@@ -219,217 +216,263 @@ gdf_debugger_client_get_state (GdfDebuggerClient *dbg)
     return ret;
 }
 
-gboolean
+GdfDebuggerClientResult
 gdf_debugger_client_load_binary (GdfDebuggerClient *dbg, 
-                                  gchar *file_name)
+                                 char *file_name)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
 
-    g_return_val_if_fail (dbg != NULL, FALSE);
-    g_return_val_if_fail (file_name != NULL, FALSE);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), FALSE);
-
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS); 
+    g_return_val_if_fail (file_name != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    
     CORBA_exception_init (&ev);
     
     GDF_Debugger_load_binary (dbg->objref,
                                file_name,
                                &ev);
 
-    /* FIXME: Check env for system error */
-    
-    if (ev._major == CORBA_USER_EXCEPTION) {
-        if (strcmp (ev._repo_id, ex_GDF_Debugger_InvalidBinary) == 0) {
-            CORBA_exception_free (&ev);   
-            return FALSE;
-        } else {
-            g_assert_not_reached ();
-        }
-    }
-
-    CORBA_exception_free (&ev);   
-
-    return TRUE;
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult 
 gdf_debugger_client_unload_binary (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
     GDF_Debugger_unload_binary (dbg->objref,
                                  &ev);
 
-    /* FIXME: Check env */
-    
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-GDF_source_file_list*
-gdf_debugger_client_get_sources (GdfDebuggerClient *dbg)
+GdfDebuggerClientResult
+gdf_debugger_client_get_sources (GdfDebuggerClient *dbg, 
+                                 GDF_source_file_list **ret)
 {
-    GDF_source_file_list *retval;
     CORBA_Environment ev;
-
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
-
+    GdfDebuggerClientResult res;
+    
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    
     CORBA_exception_init (&ev);
+    
+    *ret = GDF_Debugger_get_sources (dbg->objref,
+                                     &ev);
+    
+    res = exception_to_result (&ev);
 
-    retval = GDF_Debugger_get_sources (dbg->objref,
-                                        &ev);
-
-    /* FIXME: Check env */
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        *ret = NULL;
+    }
     
     CORBA_exception_free (&ev);
-    
-    return retval;
+    return res;
 }
 
-gchar *
+GdfDebuggerClientResult
 gdf_debugger_client_get_absolute_source_path (GdfDebuggerClient *dbg,
-                                               const gchar *file_name)
+                                              const char *file_name,
+                                              char **ret)
 {
-    gchar *retval;
     CORBA_char *corba_ret;
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
 
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (file_name != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
 
     corba_ret = GDF_Debugger_get_absolute_source_path (dbg->objref,
-                                                        file_name,
-                                                        &ev);
+                                                       file_name,
+                                                       &ev);
 
-    /* FIXME: Check env */
 
-    retval = g_strdup (corba_ret);   /* let clients use g_free rather than */
-    CORBA_free (corba_ret);          /* CORBA_free */
+    res = exception_to_result (&ev);
     
+    if (res == GDF_DEBUGGER_CLIENT_OK) {
+        /* duplicate the string so the user can g_free() */
+        *ret = g_strdup (corba_ret);
+    } else {
+        *ret = NULL;
+    }
+    
+    CORBA_free (corba_ret);
+   
     CORBA_exception_free (&ev);
-    
-    return retval;
+    return res;
 }
 
-void
+GdfDebuggerClientResult
 gdf_debugger_client_execute (GdfDebuggerClient *dbg, 
-                              int argc,
-                              char *argv[])
+                             int argc,
+                             char *argv[])
 {
     GDF_arg_list *arg_list;
-    CORBA_Environment ev;
-    
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (argv != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    CORBA_Environment ev;    
+    GdfDebuggerClientResult res;
+
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (argv != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
     
-    /* FIXME: This has not been tested */
+    /* FIXME: This isn't right */
     arg_list = GDF_arg_list__alloc();
     arg_list->_length = argc;
     arg_list->_buffer = (CORBA_char**)argv;
     
     GDF_Debugger_execute (dbg->objref,
-                           arg_list,
-                           &ev);
-
-    /* FIXME: Check env */
+                          arg_list,
+                          &ev);
 
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-void
+GdfDebuggerClientResult
 gdf_debugger_client_attach (GdfDebuggerClient *dbg, int pid)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
-    GDF_Debugger_attach (dbg->objref,
-                          pid,
-                          &ev);
-
-    /* FIXME: Check env */
+    GDF_Debugger_attach (dbg->objref, pid, &ev);
 
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-void
-gdf_debugger_client_load_corefile (GdfDebuggerClient *dbg,
-                                    const char *corefile_name)
+GdfDebuggerClientResult
+gdf_debugger_client_detach (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (corefile_name != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    
+    CORBA_exception_init (&ev);
+    
+    GDF_Debugger_detach (dbg->objref, &ev);
+
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
+}
+
+GdfDebuggerClientResult
+gdf_debugger_client_load_corefile (GdfDebuggerClient *dbg,
+                                   const char *corefile_name)
+{
+    CORBA_Environment ev;
+    GdfDebuggerClientResult res;
+
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (corefile_name != NULL, 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
     GDF_Debugger_load_corefile (dbg->objref,
-                                 (CORBA_char*)corefile_name,
-                                 &ev);
-    
-    /* FIXME: Check env */
+                                (CORBA_char*)corefile_name,
+                                &ev);
     
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-long
+GdfDebuggerClientResult
 gdf_debugger_client_set_breakpoint (GdfDebuggerClient *dbg,
-                                     const char *file_name,
-                                     long line_num,
-                                     const char *condition)
+                                    const char *file_name,
+                                    long line_num,
+                                    const char *condition,
+                                    long *bp_num)
 {
     CORBA_Environment ev;
-    long bp_num;
-    
-    g_return_val_if_fail (dbg != NULL, -1);
-    g_return_val_if_fail (file_name != NULL, -1);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), -1);
+    long num;
+    GdfDebuggerClientResult res;
+
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (file_name != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
-    bp_num = 
+    num = 
         GDF_Debugger_set_breakpoint (dbg->objref,
-                                      (CORBA_char*)(file_name ? file_name :""),
-                                      line_num,
-                                      (CORBA_char*)(condition ? condition :""),
-                                      &ev);
+                                     (CORBA_char*)(file_name ? file_name :""),
+                                     line_num,
+                                     (CORBA_char*)(condition ? condition :""),
+                                     &ev);
 
-    /* FIXME: Check env */
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        num = -1;
+    }
+
+    if (bp_num) {
+        *bp_num = num;
+    }
 
     CORBA_exception_free (&ev);
-    
-    return bp_num;
+    return res;
 }
 
-long
+GdfDebuggerClientResult
 gdf_debugger_client_set_breakpoint_function (GdfDebuggerClient *dbg,
-                                              const char *file_name,
-                                              const char *function_name,
-                                              const char *condition)
+                                             const char *file_name,
+                                             const char *function_name,
+                                             const char *condition,
+                                             long *bp_num)
 {
     CORBA_Environment ev;
-    long bp_num;
+    GdfDebuggerClientResult res;
+    long num;
     
-    g_return_val_if_fail (dbg != NULL, -1);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), -1);
-    g_return_val_if_fail (function_name != NULL, -1);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (function_name != NULL,
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
-    bp_num = 
+    num = 
         GDF_Debugger_set_breakpoint_function 
         (dbg->objref,
          (CORBA_char*)(file_name ? file_name : ""),
@@ -437,21 +480,29 @@ gdf_debugger_client_set_breakpoint_function (GdfDebuggerClient *dbg,
          (CORBA_char*)(condition ? condition : ""),
          &ev);
     
-    /* FIXME: Check env */
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        num = -1;
+    }
+
+    if (bp_num) {
+        *bp_num = num;
+    }
     
     CORBA_exception_free (&ev);
-    
-    return bp_num;
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_enable_breakpoint (GdfDebuggerClient *dbg,
-                                        long bp_num)
+                                       long bp_num)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
     
@@ -460,348 +511,387 @@ gdf_debugger_client_enable_breakpoint (GdfDebuggerClient *dbg,
                                      &ev);
 
     /* FIXMR: Check env */
-    
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_disable_breakpoint (GdfDebuggerClient *dbg,
-                                         long bp_num)
+                                        long bp_num)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
     GDF_Debugger_disable_breakpoint (dbg->objref,
                                       bp_num,
                                       &ev);
-    /* FIXME: Check env */
-    
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-void
+GdfDebuggerClientResult
 gdf_debugger_client_delete_breakpoint (GdfDebuggerClient *dbg,
-                                        long bp_num)
+                                       long bp_num)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     GDF_Debugger_delete_breakpoint (dbg->objref,
                                      bp_num,
                                      &ev);
 
-    /* FIXME: Check env */
-    
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-GDF_Breakpoint*
+GdfDebuggerClientResult
 gdf_debugger_client_get_breakpoint_info (GdfDebuggerClient *dbg,
-                                          long bp_num)
+                                         long bp_num,
+                                         GDF_Breakpoint **ret)
 {
-    GDF_Breakpoint *retval;
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
-    retval = GDF_Debugger_get_breakpoint_info (dbg->objref,
-                                                (CORBA_long)bp_num,
-                                                &ev);
- 
-    /* FIXME: Check env */
+    *ret = GDF_Debugger_get_breakpoint_info (dbg->objref,
+                                             (CORBA_long)bp_num,
+                                             &ev);
     
-    if (ev._major != CORBA_NO_EXCEPTION) {
-        CORBA_exception_free (&ev);
-        retval = NULL;
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        *ret = NULL;
     }
 
     CORBA_exception_free (&ev);
-    return retval;
+    return res;
 }
 
-GDF_RegisterList *
-gdf_debugger_client_get_registers (GdfDebuggerClient *dbg)
+GdfDebuggerClientResult
+gdf_debugger_client_get_registers (GdfDebuggerClient *dbg,
+                                   GDF_RegisterList **ret)
 {
-    GDF_RegisterList *retval;
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
-    retval = GDF_Debugger_get_registers (dbg->objref,
-                                          &ev);
+    *ret = GDF_Debugger_get_registers (dbg->objref,
+                                       &ev);
     
-    /* FIXME: Check env */
-    
-    if (ev._major != CORBA_NO_EXCEPTION) {
-        CORBA_exception_free (&ev);
-        retval = NULL;
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        *ret = NULL;
     }
 
     CORBA_exception_free (&ev);
-    return retval;    
-    
+    return res;
 }
 
 long
 gdf_debugger_client_current_frame (GdfDebuggerClient *dbg)
 {
+    g_return_val_if_fail (dbg != NULL, -1);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), -1);
+
     return dbg->priv->cur_frame;
 }
  
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_up_frame (GdfDebuggerClient *dbg)
 {
-    gdf_debugger_client_change_frame (dbg, dbg->priv->cur_frame + 1);
+    return gdf_debugger_client_change_frame (dbg, dbg->priv->cur_frame + 1);
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_down_frame (GdfDebuggerClient *dbg)
 {
-    gdf_debugger_client_change_frame (dbg, dbg->priv->cur_frame - 1);
+    return gdf_debugger_client_change_frame (dbg, dbg->priv->cur_frame - 1);
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_change_frame (GdfDebuggerClient *dbg, 
-                                   long new_id)
+                                  long new_id)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
     GDF_Debugger_change_frame (dbg->objref, new_id, &ev);
     
-    /* FIXME: Check env */
+    res = exception_to_result (&ev);
 
     CORBA_exception_free (&ev);
-    return;
+    return res;
 }
 
-GDF_StackFrame *
+GdfDebuggerClientResult
 gdf_debugger_client_get_frame (GdfDebuggerClient *dbg,
-                                long id)
+                               long id,
+                               GDF_StackFrame **ret)
 {
-    GDF_StackFrame *retval;
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
-    retval = GDF_Debugger_get_frame (dbg->objref,
-                                      id,
-                                      &ev);
+    *ret = GDF_Debugger_get_frame (dbg->objref,
+                                   id,
+                                   &ev);
 
-    /* FIXME: Check env */
-    
-    if (ev._major != CORBA_NO_EXCEPTION) {
-        CORBA_exception_free (&ev);
-        CORBA_free (retval);
-        return NULL;
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        *ret = NULL;
     }
-
+  
     CORBA_exception_free (&ev);
-    return retval;
+    return res;
 }
 
-GDF_Stack *
-gdf_debugger_client_get_backtrace (GdfDebuggerClient *dbg)
+GdfDebuggerClientResult
+gdf_debugger_client_get_backtrace (GdfDebuggerClient *dbg,
+                                   GDF_Stack **ret)
 {
-    GDF_Stack *retval;
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     
-    retval = GDF_Debugger_get_backtrace (dbg->objref, &ev);
+    *ret = GDF_Debugger_get_backtrace (dbg->objref, &ev);
 
-    /* FIXME: Check env */
-    
-    if (ev._major != CORBA_NO_EXCEPTION) {
-        CORBA_exception_free (&ev);
-        retval = NULL;
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        *ret = NULL;
     }
     
     CORBA_exception_free (&ev);
-    
-    return retval;
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_cont (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     GDF_Debugger_cont (dbg->objref,
                        &ev);
 
-    /* FIXME: Check env */
-
-    CORBA_exception_free (&ev);    
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_stop (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     GDF_Debugger_stop (dbg->objref,
                         &ev);
-    /* FIXME: Check env */
-    
-    CORBA_exception_free (&ev);    
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_restart (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     GDF_Debugger_restart (dbg->objref,
                            &ev);
 
-    /* FIXME: Check env */
-    
-    CORBA_exception_free (&ev);    
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_step_over (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     GDF_Debugger_step_over (dbg->objref,
                              &ev);
  
-    /* FIXME: Check env */
-    
-    CORBA_exception_free (&ev);    
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_step_into (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
     GDF_Debugger_step_into (dbg->objref,
-                             &ev);
+                            &ev);
 
-    /* FIXME: Check env */
-    
-    CORBA_exception_free (&ev);    
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_step_out (GdfDebuggerClient *dbg)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
    
     CORBA_exception_init (&ev);
     GDF_Debugger_step_out (dbg->objref,
-                            &ev);
-
-    /* FIXME: Check env */
+                           &ev);
     
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
-GdfSymbolSetClient *
-gdf_debugger_client_get_locals (GdfDebuggerClient *dbg)
+GdfDebuggerClientResult
+gdf_debugger_client_get_locals (GdfDebuggerClient *dbg,
+                                GdfSymbolSetClient **ret)
 {
     CORBA_Environment ev;
-    GdfSymbolSetClient *retval;
     GDF_SymbolSet set;
+    GdfDebuggerClientResult res;
     
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
    
     CORBA_exception_init (&ev);
     set = GDF_Debugger_get_locals (dbg->objref, &ev);
 
-    /* FIXME: Check env */
-    
-    retval = gdf_symbol_set_client_new_from_corba (set);
+    res = exception_to_result (&ev);
+    if (res == GDF_DEBUGGER_CLIENT_OK) {
+        *ret = gdf_symbol_set_client_new_from_corba (set);
+    } else {
+        CORBA_Object_release (set, &ev);
+        *ret = NULL;
+    }
 
     CORBA_exception_free (&ev);
-
-    return retval;
+    return res;
 }   
 
-GdfSymbolSetClient *
-gdf_debugger_client_allocate_symbol_set (GdfDebuggerClient *dbg)
+GdfDebuggerClientResult
+gdf_debugger_client_allocate_symbol_set (GdfDebuggerClient *dbg,
+                                         GdfSymbolSetClient **ret)
 {    
     CORBA_Environment ev;
-    GdfSymbolSetClient *retval;
     GDF_SymbolSet set;
+    GdfDebuggerClientResult res;
     
-    g_return_val_if_fail (dbg != NULL, NULL);
-    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), NULL);
+    g_return_val_if_fail (dbg != NULL, 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
    
     CORBA_exception_init (&ev);
     set = GDF_Debugger_allocate_symbol_set (dbg->objref, &ev);
 
-    /* FIXME: Check env */
+    res = exception_to_result (&ev);
+    if (res == GDF_DEBUGGER_CLIENT_OK) {
+        *ret = gdf_symbol_set_client_new_from_corba (set);
+    } else {
+        CORBA_Object_release (set, &ev);
+        *ret = NULL;
+    }
 
-    retval = gdf_symbol_set_client_new_from_corba (set);
-    
     CORBA_exception_free (&ev);
-
-    return retval;
+    return res;
 }
 
-void 
+GdfDebuggerClientResult
 gdf_debugger_client_set_output_tty (GdfDebuggerClient *dbg,
-                                     const char *tty_name)
+                                    const char *tty_name)
 {
     CORBA_Environment ev;
+    GdfDebuggerClientResult res;
     
-    g_return_if_fail (dbg != NULL);
-    g_return_if_fail (tty_name != NULL);
-    g_return_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg));
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (tty_name != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
     CORBA_exception_init (&ev);
 
     GDF_Debugger_set_output_tty (dbg->objref, (CORBA_char*)tty_name, &ev);
     
-    /* FIXME: Check env */
+    res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
+    return res;
 }
 
 /* private routines */
@@ -876,6 +966,23 @@ debugger_client_class_init (GdfDebuggerClientClass *klass)
                         gtk_marshal_NONE__INT,
                         GTK_TYPE_NONE, 1,
                         GTK_TYPE_INT);
+    debugger_client_signals [ATTACHED] = 
+        gtk_signal_new ("attached",
+                        GTK_RUN_FIRST,
+                        object_class->type,
+                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
+                                           attached),
+                        gtk_marshal_NONE__NONE,
+                        GTK_TYPE_NONE, 0);
+    debugger_client_signals [DETACHED] = 
+        gtk_signal_new ("detached",
+                        GTK_RUN_FIRST,
+                        object_class->type,
+                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
+                                           detached),
+                        gtk_marshal_NONE__NONE,
+                        GTK_TYPE_NONE, 0,
+                        GTK_TYPE_NONE);
     debugger_client_signals [EXECUTION_STARTED] = 
         gtk_signal_new ("execution_started",
                         GTK_RUN_FIRST,
@@ -960,8 +1067,8 @@ debugger_client_class_init (GdfDebuggerClientClass *klass)
                                   debugger_client_signals,
                                   LAST_SIGNAL);
 
-    klass->program_loaded = program_loaded_impl;
-    klass->program_unloaded = program_unloaded_impl;
+    klass->program_loaded = NULL;
+    klass->program_unloaded = NULL;
     klass->breakpoint_set = NULL;
     object_class->destroy = (GtkSignalFunc) debugger_client_destroy;
 }
@@ -989,9 +1096,7 @@ debugger_client_destroy (GdfDebuggerClient *client)
 
     CORBA_exception_init (&ev);
     
-    Bonobo_Unknown_unref (client->objref, &ev);
-    
-    /* FIXME: Check env */
+    Bonobo_Unknown_unref (client->objref, &ev);   
     
     CORBA_exception_free (&ev);
 
@@ -1032,6 +1137,12 @@ emit_event_signal (GdfDebuggerClient *dbg,
         gtk_signal_emit (GTK_OBJECT (dbg),
                          debugger_client_signals[BREAKPOINT_DELETED],
                          (gint)event->argument.long_arg);
+    } else if (strcmp (event->event_name, "attached") == 0) {
+        gtk_signal_emit (GTK_OBJECT (dbg),
+                         debugger_client_signals[ATTACHED]);
+    } else if (strcmp (event->event_name, "detached") == 0) {
+        gtk_signal_emit (GTK_OBJECT (dbg),
+                         debugger_client_signals[DETACHED]);
     } else if (strcmp (event->event_name, "started") == 0) {
         gtk_signal_emit (GTK_OBJECT (dbg),
                          debugger_client_signals[EXECUTION_STARTED]);
@@ -1090,17 +1201,42 @@ event_pushed_cb (GdfEventChannelClient *event_channel,
     gdf_event_destroy (event);
 }
 
-void
-program_loaded_impl (GdfDebuggerClient *dbg) 
+static GdfDebuggerClientResult 
+exception_to_result (CORBA_Environment *ev)
 {
-    dbg->program_loaded = TRUE;
+    GdfDebuggerClientResult res;
+    static GHashTable *table = NULL;
+
+    if (!table) {
+        table = g_hash_table_new (g_str_hash, g_str_equal);      
+        g_hash_table_insert (table, ex_GDF_Debugger_InvalidBinary,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_INVALID_BINARY));
+        g_hash_table_insert (table, ex_GDF_Debugger_NoSymbols,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NO_SYMBOLS));
+        g_hash_table_insert (table, ex_GDF_Debugger_DoesntExist,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_DOESNT_EXIST));
+        g_hash_table_insert (table, ex_GDF_Debugger_NoBinary,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NO_BINARY));
+        g_hash_table_insert (table, ex_GDF_Debugger_NotExecutable,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NOT_EXECUTABLE));
+        g_hash_table_insert (table, ex_GDF_Debugger_NotAttached,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NOT_ATTACHED));
+        g_hash_table_insert (table, ex_GDF_Debugger_NotRunning,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NOT_RUNNING));
+        g_hash_table_insert (table, ex_GDF_Debugger_SecurityException,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_SECURITY_EXCEPTION));
+        g_hash_table_insert (table, ex_GDF_Debugger_InvalidState,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_INVALID_STATE));
+        g_hash_table_insert (table, ex_GDF_Debugger_UnknownError,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_UNKNOWN_ERROR));
+    }
+
+    res = GDF_DEBUGGER_CLIENT_OK;
+    
+    if (ev->_major == CORBA_USER_EXCEPTION) {
+        res = GPOINTER_TO_INT (g_hash_table_lookup (table, ev->_repo_id));
+    }
+
+    return res;
 }
 
-void
-program_unloaded_impl (GdfDebuggerClient *dbg)
-{
-    dbg->program_loaded = FALSE;
-}
-
-
-
